<?php

/**
 * Plugin Name: Enbank Payment for Woocommerce
 * Author Name: inflabs0
 * Description: This plugin allows woocommerce users to execute account to account payments.
 * Version: 0.1.0
 * License: 0.1.0
 * License URL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * text-domain: enbank-pay-woo
*/
header("Access-Control-Allow-Origin: *");

use function PHPSTORM_META\map;

if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) return;

add_action( 'plugins_loaded', 'enbank_payment_init', 11 );

function enbank_payment_init() {
    if( class_exists( 'WC_Payment_Gateway' ) ) {
        class Enbank_Payment_Gateway extends WC_Payment_Gateway {
            public function __construct() {
                $plugin_dir = plugin_dir_url(__FILE__);
                $this->id   = 'enbank_payment';
                $this->icon = apply_filters( 'woocommerce_gateway_icon', $plugin_dir.'\assets\icon_enbank.png' );
                $this->has_fields = false;
                $this->method_title = __( 'Enbank Payment', 'enbank-pay-woo');
                $this->method_description = __( 'Enbank payment systems.', 'enbank-pay-woo');

                $this->init_form_fields();
                $this->init_settings();

                $this->title = $this->get_option('title');
                $this->description = $this->get_option('description');
                $this->instructions = $this->get_option( 'instructions', $this->description );
                $this->token = $this->get_option('token');
                $this->enbank_url = $this->get_option('enbank_url');

                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
                add_action( 'woocommerce_api_enbank', array( $this, 'webhook' ) );
                // add_action( 'woocommerce_thank_you_' . $this->id, array( $this, 'thank_you_page' ) );
            }

            public function init_form_fields() {
                $this->form_fields = apply_filters( 'woo_enbank_pay_fields', array(
                    'enabled' => array(
                        'title' => __( 'Enable/Disable', 'enbank-pay-woo'),
                        'type' => 'checkbox',
                        'label' => __( 'Enable or Disable Enbank Payments', 'enbank-pay-woo'),
                        'default' => 'no'
                    ),
                    'title' => array(
                        'title' => __( 'Enbank Payments Gateway', 'enbank-pay-woo'),
                        'type' => 'text',
                        'default' => __( 'Плати със своята банка', 'enbank-pay-woo'),
                        'desc_tip' => true,
                        'description' => __( 'Add a new title for the Enbank Payments Gateway that customers will see when they are in the checkout page.', 'enbank-pay-woo')
                    ),
                    'description' => array(
                        'title' => __( 'Enbank Payments Gateway Description', 'enbank-pay-woo'),
                        'type' => 'textarea',
                        'default' => __( 'Платете бързо и сигурно с няколко клика през вашето онлайн банкиране', 'enbank-pay-woo'),
                        'desc_tip' => true,
                        'description' => __( 'Add a new title for the Enbank Payments Gateway that customers will see when they are in the checkout page.', 'enbank-pay-woo')
                    ),
                    'instructions' => array(
                        'title' => __( 'Instructions', 'enbank-pay-woo'),
                        'type' => 'textarea',
                        'default' => __( 'Default instructions', 'enbank-pay-woo'),
                        'desc_tip' => true,
                        'description' => __( 'Instructions that will be added to the thank you page and odrer email', 'enbank-pay-woo')
                    ),
                    'token' => array(
                        'title' => __( 'Merchan Token', 'enbank-pay-woo'),
                        'type' => 'text',
                        'default' => __( 'cl8df0puw0001d8u0qi258awk', 'enbank-pay-woo'),
                        'desc_tip' => true,
                        'description' => __( 'Merchant token', 'enbank-pay-woo')
                    ),
                    'enbank_url' => array(
                        'title' => __( 'Enbank Url', 'enbank-pay-woo'),
                        'type' => 'text',
                        'default' => __( 'https://enbank-server-local.fly.dev', 'enbank-pay-woo'),
                        'desc_tip' => true,
                        'description' => __( 'Enbank url', 'enbank-pay-woo')
                    ),
                ));
            }

            public function process_payment( $order_id ) {
                global $woocommerce;
	           
	            $order = wc_get_order( $order_id );

                $order->update_status('pending', __( 'Awaiting Enbank payment', 'enbank-pay-woo' ));  

                $amount = $order->get_total();
                $description = 'This is test payment from wordpress';

                $items = array();
                foreach ( $order->get_items() as $item_id => $item ) {
                    $product = wc_get_product( $item['product_id'] );
                    $id = $product->id;
                    $am = array("discount" => 0, "subtotal" => 0, "vat" => 0, "vatCoefficient" => 0, "total" => $product->price);
                    $currency = 'BGN';
                    $description = htmlspecialchars_decode($product->name);
                    $quantity = $item->get_quantity(); 
                    
                    $itm = array(
                        'id' => $id,
                        'amount' => $am,
                        'currency' => $currency,
                        'description' => $description,
                        'quantity' => $quantity  
                    );
                    array_push($items, $itm);
                }

                $success_url = add_query_arg( array(
                    'wc-api' => 'enbank',
                    'id' => json_encode($order_id)
                    
                ), get_site_url() );

                $failure_url = add_query_arg( array(
                    'wc-api' => 'enbank',
                    'id' => json_encode(-$order_id)
                    
                ), get_site_url() );

                $new_transaction_url = esc_url($this->enbank_url . '/transactions/new');

                $transaction_id =  enbank_create_transaction($amount, $this->token, $description, $items, $success_url, $failure_url, $new_transaction_url, $this->get_return_url( $order ));

                $woocommerce->cart->empty_cart();

                return array(
                    'result' => 'success',
                    'redirect' => esc_url($this->enbank_url . "/checkout/$transaction_id")
                );
            }

            public function thank_you_page(){
                if( $this->instructions ){
                    echo wpautop(esc_html($this->instructions));
                }
            }

            public function webhook() {
                $order_id = intval($_GET['id']);
                if ($order_id < 0) {
                    $order = wc_get_order( -$order_id );
                    $order->update_status( 'failed' );
                } else {
                    $order = wc_get_order( $order_id );
                    $order->update_status( 'completed' );
                }
                // $order->reduce_order_stock();
	            //update_option('webhook_debug', $_GET);
            }
        }
    }
}

add_filter( 'woocommerce_payment_gateways', 'enbank_add_to_woo_payment_gateway');

function enbank_add_to_woo_payment_gateway( $gateways ) {
    $gateways[] = 'Enbank_Payment_Gateway';
    return $gateways;
}

function enbank_create_transaction($amount, $token, $description, $items, $scallback, $fcallback, $url, $receivedOrderUrl) {

    $body = json_encode([
        'amount' => $amount,
        'token' => $token,
        'description' => $description,
        'items' => [],
        'successCallback' => $scallback,
        'errorCallback' => $fcallback,
        'flags' => [
            "lineItems" => $items,
            "receivedOrderUrl" => $receivedOrderUrl
        ]
        ]);
    
    $response = wp_remote_post( $url , array (
        'method' => 'POST',
        'blocking' => true,
        'data_format' => 'body',
        'headers'     => array('Content-Type' => 'application/json; charset=utf-8'),
        'body'        => $body
    ));
    
    $transactionLog = fopen("transactionLog.txt", "w") or die("Unable to open file!");
   
    if( is_wp_error($response )) {
        fwrite($transactionLog, 'ERROR: ' . $response->get_error_message() . "\n");
    } else {    
        fwrite($transactionLog, 'SUCCESS: ' . json_decode($response['body'])->transactionId . "\n");
        return json_decode($response['body'])->transactionId;
    }
    fflush($transactionLog);
    fclose($transactionLog);

}

// add_filter( 'woocommerce_available_payment_gateways', 'woocommerce_available_payment_gateways' );
// function woocommerce_available_payment_gateways( $available_gateways ) {
//     if (! is_checkout() ) return $available_gateways;  // stop doing anything if we're not on checkout page.
//     if (array_key_exists('enbank_payment',$available_gateways)) { 
//          $available_gateways['enbank_payment']->order_button_text = __( 'Pay with Enbank', 'woocommerce' );
//     }
//     return $available_gateways;
// }